"""
Generic database that uses ZODB.
"""

#*****************************************************************************
#       MANIN: Modular Forms Software, (c) William Stein, 2004
#
#       Copyright (C) 2004 William Stein <was@math.harvard.edu>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  http://www.gnu.org/licenses/
#*****************************************************************************

import os
import ZODB, BTree
from ZODB import FileStorage, DB

DB_HOME = "%s/src/data/zodb/"%os.environ["SAGE_HOME"]

_db = {}
class _uniq(object):
    def __new__(cls, read_only=True):
        global _db
        if _db.has_key(cls):
            return _db[cls]
        X = object.__new__(cls)
        _db[cls] = X
        return X

class Database(_uniq):
    def __init__(self, name, read_only=True):
        if not hasattr(self, 'name'):
            self.name = name
            if not os.path.exists("%s/%s"%(DB_HOME,name)):
                os.makedirs("%s/%s"%(DB_HOME,name))
            self._storage = FileStorage.FileStorage("%s/%s/%s"%(
                DB_HOME, name, name), read_only=read_only)
            self._db = DB(self._storage)
            self.conn = self._db.open()
            self._root = self.conn.root()
            if not self._root.has_key("btree"):
                self._root["btree"] = BTree.BTree()
            self.root = self._root["btree"]

    def abort(self):
        get_transaction().abort()

    def commit(self):
        self.root._p_changed = 1
        get_transaction().commit()

    def pack(self):
        self._db.pack()
        self.commit()

    def __repr__(self):
        return "Database %s"%self.name

    def __setitem__(self, x, y):
        self.root[x] = y

    def __getitem__(self, x):
        return self.root[x]

    def has_key(self, x):
        return self.root.has_key(x)

    def keys(self):
        return self.root.keys()
